module Appforce
  class Config
    require 'yaml'
    require 'ostruct'

    # read in config file
    # default location is root at runtime
    DEFAULT_CONFIG_PATH = "#{File.expand_path('~')}/.appforce"

    def self.load_config(path=nil)
      begin
        temp = YAML::load(File.open(path.nil? ? DEFAULT_CONFIG_PATH : path))
        @config = OpenStruct.new temp
      rescue Exception => e
        logger.fatal "[#{self.name}##{__method__.to_s}] Appforce::Spawn::ConfigFileParseError"\
          " -- Config file appears to be malformed -- #{e}"
        raise e
      end  
    end

    def self.config
      @config
    end

    def self.dump_example_config
      path = "#{File.expand_path('~')}/.appforce.example"
      h = {
        'api_host'    => 'https://afuka.synctree.com',
        'api_version' => 'api/v1',
        'api_token'   => 'PUT_API_TOKEN_HERE'
      }
      begin
        file = File.open(path, "w")
        file.write(h.to_yaml)
      rescue IOError => e
        logger.fatal "[#{self.name}##{__method__.to_s}] Appforce::ConfigDumpError -- #{e}"
        raise e
      ensure
        file.close unless file == nil
      end
    end

    private

    def self.logger
      Appforce::Logger.logger
    end

  end
end