module Appforce
  module Spawn
    module Api
      class Call
        require 'highline/import'

        def self.get_clients
          url = "#{base_api_url}client/list?#{token_param}"
          logger.debug "[#{self.name}##{__method__.to_s}] URL: #{url}"
          make_api_call(url)
        end
        
        def self.get_hosts(*args)
          opts = args[1]
          unless opts[:client_api_name]
            logger.error "[#{self.name}##{__method__.to_s}] Appforce::Spawn::MissingParameters -- Missing Client API Name"
            raise Appforce::Spawn::MissingParameters, "Missing Client API Name"
          end

          url = "#{client_url(opts[:client_api_name])}/hosts?#{token_param}"
          logger.debug "[#{self.name}##{__method__.to_s}] URL: #{url}"
          make_api_call(url)
        end

        def self.get_active_users(*args)
          opts = args[1]
          unless opts[:client_api_name]
            logger.error "[#{self.name}##{__method__.to_s}] Appforce::Spawn::MissingParameters -- Missing Client API Name"
            raise Appforce::Spawn::MissingParameters, "Missing Client API Name"
          end

          url = "#{client_url(opts[:client_api_name])}/users/active.yml?#{token_param}"
          logger.debug "[#{self.name}##{__method__.to_s}] URL: #{url}"
          make_api_call(url)
        end

        def self.get_inactive_users(*args)
          opts = args[1]
          unless opts[:client_api_name]
            logger.error "[#{self.name}##{__method__.to_s}] Appforce::Spawn::MissingParameters -- Missing Client API Name"
            raise Appforce::Spawn::MissingParameters, "Missing Client API Name"
          end

          url = "#{client_url(opts[:client_api_name])}/users/inactive.yml?#{token_param}"
          logger.debug "[#{self.name}##{__method__.to_s}] URL: #{url}"
          make_api_call(url)
        end

        def self.get_vars(*args)
          opts = args[1]
          unless opts[:client_api_name]
            logger.error "[#{self.name}##{__method__.to_s}] Appforce::Spawn::MissingParameters -- Missing Client API Name"
            raise Appforce::Spawn::MissingParameters, "Missing Client API Name"
          end
          
          url = "#{client_url(opts[:client_api_name])}/vars.yml?#{token_param}"
          logger.debug "[#{self.name}##{__method__.to_s}] URL: #{url}"
          make_api_call(url)
        end

        def self.get_main_scout(*args)
          opts = args[1]
          unless opts[:client_api_name]
            logger.error "[#{self.name}##{__method__.to_s}] Appforce::Spawn::MissingParameters -- Missing Client API Name"
            raise Appforce::Spawn::MissingParameters, "Missing Client API Name"
          end
          
          url = "#{client_url(opts[:client_api_name])}/scout.yml?#{token_param}"
          logger.debug "[#{self.name}##{__method__.to_s}] URL: #{url}"
          make_api_call(url)
        end

        def self.get_private_key(*args)
          opts = args[1]
          unless opts[:client_api_name]
            logger.error "[#{self.name}##{__method__.to_s}] Appforce::Spawn::MissingParameters -- Missing Client API Name"
            raise Appforce::Spawn::MissingParameters, "Missing Client API Name"
          end

          url = "#{client_url(opts[:client_api_name])}/private_key.pem?#{token_param}"
          logger.debug "[#{self.name}##{__method__.to_s}] URL: #{url}"
          make_api_call(url)   
        end

        def self.get_host_scout_vars(*args)
          opts = args[1]
          unless opts[:host_api_name]
            logger.error "[#{self.name}##{__method__.to_s}] Appforce::Spawn::MissingParameters -- Missing Host API Name"
            raise Appforce::Spawn::MissingParameters, "Missing Host API Name"
          end
          
          url = "#{host_url(opts[:host_api_name])}/scout.yml?#{token_param}"
          logger.debug "[#{self.name}##{__method__.to_s}] URL: #{url}"
          make_api_call(url)
        end

        def self.get_all_host_data(*args)
          opts = args[1]
          unless opts[:client_api_name]
            logger.error "[#{self.name}##{__method__.to_s}] Appforce::Spawn::MissingParameters -- Missing Client API Name"
            raise Appforce::Spawn::MissingParameters, "Missing Client API Name"
          end
          
          url = "#{client_url(opts[:client_api_name])}/all_host_data.json?#{token_param}"
          logger.debug "[#{self.name}##{__method__.to_s}] URL: #{url}"
          make_api_call(url)
        end

        def self.list_clients
          resp = get_clients
          puts Appforce::Logger.header("Client List")
          resp.each do |item|
            printf " Client: %-16s API Name: %s\n", item['name'], item['api_name']
          end
          puts Appforce::Logger.footer
        end

        def self.list_hosts(*args)
          resp = get_all_host_data(*args)
          puts Appforce::Logger.header("Host List")
          i = 1
          resp.each do |item|
            printf " #{i} - '%s' host: %s ip: %s\n", item['desc'], item['hostname'], item['ip']
            i += 1
          end
          puts Appforce::Logger.footer
          resp
        end

        def self.ssh_to_host(*args)
          hosts   = list_hosts(*args)
          input   = HighLine.ask("\nWhich host to ssh to? ", Integer) { |q| q.in = 1..hosts.count }
          machine = hosts[input - 1]
          
          $stderr.print "\n"
          logger.info "Establishing SSH seesion as '#{machine['username']}' to #{machine['desc']}"
          logger.debug "SSH command: #{ssh_cmd(machine)}"

          pid = fork {
            exec ssh_cmd(machine)
          }
          Process.wait pid
          
          $stderr.print "\n"
          logger.info "SSH session terminated."
        end

        def self.ping(*args)
          url = "#{base_api_url}ping?#{token_param}"
          logger.debug "[#{self.name}##{__method__.to_s}] URL: #{url}"
          make_api_call("#{base_api_url}ping?#{token_param}")
        end

        def self.latest_version(*args)
          url = rubygem_latest_version_url
          logger.debug "[#{self.name}##{__method__.to_s}] URL: #{url}"
          resp = make_api_call(url)
          resp['version']
        end
        
        private

        def self.ssh_cmd(info)
          "ssh #{info['username']}@#{info[info['method']]}"
        end

        def self.base_api_url
          "#{Appforce::Config.config.api_host}/#{Appforce::Config.config.api_version}/"
        end

        def self.client_url(client_api_name)
          "#{base_api_url}client/#{client_api_name}"
        end

        def self.host_url(host_api_name)
          "#{base_api_url}host/#{host_api_name}"
        end

        def self.token_param
          "token=#{Appforce::Config.config.api_token}"
        end

        def self.rubygem_latest_version_url
          "https://rubygems.org/api/v1/versions/appforce-spawn/latest.json"
        end
        
        def self.make_api_call(url)
          begin
            resp = HTTParty.get(url)
          rescue
            logger.fatal "[#{self.name}##{__method__.to_s}] Appforce::Spawn::API::CallFailure - API Failed to Return for URL: #{url}"
            raise Appforce::Spawn::API::CallFailure, "API Failed to Return."
          end

          case resp.code
            when 200
              resp
            when 401
              logger.fatal "[#{self.name}##{__method__.to_s}] Appforce::Spawn::API::UnauthorizedAccess -- Invalid API Access Token -- URL: #{url}"
              raise Appforce::Spawn::API::UnauthorizedAccess, "Invalid API Access Token"
            when 404
              logger.fatal "[#{self.name}##{__method__.to_s}] -- Appforce::Spawn::API::NotFound -- API Route Not Found -- URL: #{url}"
              raise Appforce::Spawn::API::NotFound, "API Route Not Found"
            when 500...600
              logger.fatal "[#{self.name}##{__method__.to_s}] -- Appforce::Spawn::API::Unavailable -- API Service Unavailable -- URL: #{url}"
              raise Appforce::Spawn::API::Unavailable, "API Service Unavailable"
          end
        end

        def self.logger
          Appforce::Logger.logger
        end

      end
    end
  end
end
